function Peq = eqprobvect(T,dofix)

% calculate the equilibrium probability distribution given the transition
% rate matrix T.
% In general having rates specified in per second for gene activity
% transitions will be slow enough that we don't get multiple transitions
% within one frame, but in case that is possible, apply a fix which scales
% all the rates to be arbitrarily small

if nargin<2 || isempty(dofix)
    dofix = 0; % don't apply the fix by default
end

if dofix
    % scale so that the largest transition is 0.01
    
    % set the null transitions to zero before summing
    T(logical(eye(size(T)))) = 0;
    
    maxrate = max(sum(T,2));
    temp = T/maxrate * 0.01;
    
    T = temp + diag(1-sum(temp,2),0);
end

% % [v,d] = eig(T');
% % ddiff = abs(diag(d)-1);
% % 
% % Peq = v(:,find(ddiff==min(ddiff),1,'first'));

if size(T,1)>1
    try
    [Peq,tempd] = eigs(T',1,1);
    catch
        Peq = ones(size(T,1),1)/size(T,1);
    end
else
    Peq = 1;
end

Peq = abs(Peq)/sum(abs(Peq));

