classdef timerBar < handle
    properties (Hidden)
        wbh = NaN;
        tich = [];
    end
    properties (SetAccess = private)
        prog = 0;
        timedata = [];
        avframes = 10;
        numFrames = NaN;
        framesLeft = NaN;
        timerem = NaN;
        msg
        showbar = true;
    end
    
    methods
        function TB = timerBar(msg,nframes,hide)
            % need to have a way of initializing without starting running
            if nargin<3 || isempty(hide)
                hide = 0;
            end
            if nargin<1 || isempty(msg)
                msg = '';
            end
            fullmsg = 'Initializing';
            if ~isempty(msg)
                fullmsg = [msg ':' fullmsg];
            end
            
            TB.showbar = hide==0;
            if TB.showbar
                TB.wbh = adwaitbar(NaN,0,fullmsg);
            end
            TB.msg = msg;
            
            % if we've supplied the number of frames then start running
            if nargin>1 && ~isempty(nframes)
                TB.start(nframes)
            end
        end
        function start(TB,nframes)
            % start the first timer running
            if nargin>1
                TB.numFrames = nframes;
                TB.framesLeft = nframes;
            end
%             if isnan(TB.numFrames)
%                 error('Number of frames not initialized')
%             end
            
            TB.tich = tic;
            
            % change the waitbar message to show it's running.
            fullmsg = 'Calculating time left';
            if ~isempty(TB.msg)
                fullmsg = [TB.msg ':' fullmsg];
            end
            
            if TB.showbar
                TB.wbh = adwaitbar(TB.wbh,0,fullmsg);
            end
            
        end
        function setav(TB,avframes)
            TB.avframes = avframes;
        end
        function updatemsg(TB,msg)
            if nargin>1 && ~isempty(msg)
                TB.msg = msg;
            end
            fullmsg = timeremmsg(TB.timerem);
            
            if ~isempty(TB.msg)
                fullmsg = [TB.msg,':',fullmsg];
            end
            if TB.showbar
                TB.wbh = adwaitbar(TB.wbh,TB.prog,fullmsg);
            end
            
        end
        function increment(TB,step,nframes)
            % update the timerbar by step units
            
            % allow the total number of frames to be specified now at the
            % latest
            if nargin>2 && ~isempty(nframes)
                % allow a change in nframes to be reflected in the time
                % remaining
                prevframes = TB.numFrames;
                
                TB.numFrames = nframes;
                
                if isnan(prevframes)
                    TB.framesLeft = nframes;
                else
                    % change framesleft by the same amount as nframes has
                    % changed
                    TB.framesLeft = TB.framesLeft + (nframes-prevframes);
                end
            end
            if isnan(TB.numFrames)
                % don't render anything on the bar till we know..
                return
            end
            
            if nargin<2 || isempty(step)
                step = 1;
            end
            
            TB.framesLeft = TB.framesLeft - step;
            
            if TB.framesLeft>0
                tempt = toc(TB.tich);
                TB.tich = tic;
                
                TB.timedata = [TB.timedata, ones(1,step)*tempt/step];
                TB.timedata = TB.timedata(max(1,end-TB.avframes+1):end);
                
                spf = median(TB.timedata);
                TB.timerem = TB.framesLeft*spf;
                TB.prog = 1 - TB.framesLeft/TB.numFrames;
                
                fullmsg = timeremmsg(TB.timerem);
                if ~isempty(TB.msg)
                    fullmsg = [TB.msg,':',fullmsg];
                end
                
                if TB.showbar
                    TB.wbh = adwaitbar(TB.wbh,TB.prog,fullmsg);
                end
            else
                TB.prog = 1;
                TB.timerem = 0;
                fullmsg = 'Finishing';
                if ~isempty(TB.msg)
                    fullmsg = [TB.msg,':',fullmsg];
                end
                
                if TB.showbar
                    TB.wbh = adwaitbar(TB.wbh,1,fullmsg);
                end
            end
            
        end
        
        function finish(TB)
            if ishandle(TB.wbh)
                close(TB.wbh)
            end
        end
        
        function hidebar(TB)
            TB.showbar = false;
            
            if ishandle(TB.wbh)
                delete(TB.wbh)
            end
        end
        
        function unhidebar(TB)
            TB.showbar = true;
            TB.wbh = adwaitbar(TB.wbh,TB.prog,fullmsg);
        end
    end
    
end